'use strict';

var _class; /**
             *
             * The MIT License (MIT)
             *
             * Copyright (c) 2015 Invertase
             *
             * Permission is hereby granted, free of charge, to any person obtaining a copy
             * of this software and associated documentation files (the "Software"), to deal
             * in the Software without restriction, including without limitation the rights
             * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
             * copies of the Software, and to permit persons to whom the Software is
             * furnished to do so, subject to the following conditions:
             *
             * The above copyright notice and this permission notice shall be included in all
             * copies or substantial portions of the Software.
             *
             * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
             * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
             * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
             * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
             * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
             * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
             * SOFTWARE.
             *
             */

// TODO  - listenable decorator is working, just need to setup event emits and improve emitter logic.

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _listenable = require('./listenable');

var _listenable2 = _interopRequireDefault(_listenable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let Database = (0, _listenable2.default)(_class = class Database extends Map {

  constructor(id) {
    super();
    this.id = id; // TODO unused for now
    this.watchers = new WeakMap();
    this.timers = new WeakMap();
    return this;
  }

  flushdb() {
    super.clear();
    return 1;
  }

  set(key, value) {
    // this.emit(this, key, value); // todo
    super.set(key, value);
    return 1;
  }

  get(key) {
    return super.get(key) || null;
  }

  setnx(key, value) {
    if (super.has(key)) {
      return 0;
    }
    return this.set(key, value);
  }

  append(key, value) {
    const type = this.type(key);
    if (type === 'none') {
      super.set(key, value);
      return value.length;
    }
    if (type === 'string') {
      const currentValue = super.get(key);
      super.set(key, currentValue + value);
      return (currentValue + value).length;
    }
    return -1; // key not a valid type for append.
  }

  strlen(key) {
    const value = super.get(key);
    return value ? value.length : 0;
  }

  exists(key) {
    if (super.has(key)) {
      return 1;
    }
    return 0;
  }

  type(key) {
    if (super.has(key)) {
      const value = super.get(key);
      if (value instanceof Map) {
        return 'hash';
      }
      return Array.isArray(value) ? 'array' : typeof value;
    }
    return 'none';
  }

  del(key) {
    if (arguments.length > 2 || Array.isArray(key)) {
      let deleted = 0;
      const keys = Array.isArray(key) ? key : [...arguments].slice(1, arguments.length);

      if (!(keys && typeof keys[Symbol.iterator] === 'function')) {
        throw new TypeError('Expected keys to be iterable, got ' + (keys === null ? 'null' : typeof keys === 'object' && keys.constructor ? keys.constructor.name || '[Unknown Object]' : typeof keys));
      }

      for (const _key of keys) {
        if (_key) {
          deleted = deleted + this.del(_key);
        }
      }
      return deleted;
    }
    if (super.delete(key)) {
      return 1;
    }
    return 0;
  }

  hget(hash, field) {
    if (super.has(hash)) {
      try {
        return super.get(hash).get(field) || null;
      } catch (ignoreMe) {
        return null;
      }
    }
    return null;
  }

  hset(hash, field, value) {
    try {
      super.get(hash).set(field, value);
      return 1;
    } catch (ignoreMe) {
      super.set(hash, new Map([[field, value]]));
      return 1;
    }
  }

  hsetnx(hash, field, value) {
    try {
      if (super.get(hash).has(field)) {
        return 0;
      }
      return this.hset(...arguments);
    } catch (ignoreMe) {
      return this.hset(...arguments);
    }
  }

  hdel(hash, field) {
    if (arguments.length > 2 || Array.isArray(field)) {
      let deleted = 0;
      const fields = Array.isArray(field) ? field : [...arguments].slice(1, arguments.length);

      if (!(fields && typeof fields[Symbol.iterator] === 'function')) {
        throw new TypeError('Expected fields to be iterable, got ' + (fields === null ? 'null' : typeof fields === 'object' && fields.constructor ? fields.constructor.name || '[Unknown Object]' : typeof fields));
      }

      for (const _field of fields) {
        if (_field) {
          deleted = deleted + this.hdel(hash, _field);
        }
      }
      return deleted;
    }
    try {
      if (super.get(hash).delete(field)) {
        return 1;
      }
      return 0;
    } catch (ignoreMe) {
      return 0;
    }
  }

  hkeys(hash) {
    try {
      const keys = super.get(hash).keys();
      return [...keys];
    } catch (ignoreMe) {
      return [];
    }
  }

  hvals(hash) {
    try {
      const values = super.get(hash).values();
      return [...values];
    } catch (ignoreMe) {
      return [];
    }
  }

  hlen(hash) {
    try {
      return super.get(hash).size || 0;
    } catch (ignoreMe) {
      return 0;
    }
  }

  hexists(hash, field) {
    try {
      if (super.get(hash).has(field)) {
        return 1;
      }
      return 0;
    } catch (ignoreMe) {
      return 0;
    }
  }

  hgetall(hash) {
    try {
      const entries = super.get(hash).entries();
      return [...entries];
    } catch (ignoreMe) {
      return [];
    }
  }

  hstrlen(hash, field) {
    const value = this.hget(hash, field);
    return value ? value.length : 0;
  }
}) || _class;

exports.default = Database;
module.exports = exports['default'];