"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (Class) {

  Object.defineProperties(Class.prototype, {

    getListenable: {
      value: function (object) {
        if (!this.listeners.has(object)) {
          this.listeners.set(object, {});
        }
        return this.listeners.get(object);
      },
      enumerable: true
    },

    removeListener: {
      value: function (object, identifier, listener) {
        const listeners = getListeners(object, identifier);
        const index = listeners.indexOf(listener);
        if (index !== -1) {
          listeners.splice(index, 1);
        }
      }
    },

    getListeners: {
      value: function (object, identifier) {
        const listenable = this.getListenable(object);
        listenable[identifier] = listenable[identifier] || [];
        return listenable[identifier];
      },
      enumerable: true
    },

    on: {
      value: function (object, identifier, listener) {
        const listeners = this.getListeners(object, identifier);
        listeners.push(listener);
      },
      enumerable: true
    },

    emit: {
      value: function (object, identifier) {
        if (this.listeners.size > 0) {
          const listeners = this.getListeners(object, identifier);

          if (!(listeners && typeof listeners[Symbol.iterator] === 'function')) {
            throw new TypeError("Expected listeners to be iterable, got " + (listeners === null ? 'null' : typeof listeners === 'object' && listeners.constructor ? listeners.constructor.name || '[Unknown Object]' : typeof listeners));
          }

          for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
            args[_key - 2] = arguments[_key];
          }

          for (const listener of listeners) {
            listener.apply(object, args);
          }
        }
      }
    },

    listeners: {
      value: new WeakMap()
    }

  });
};

module.exports = exports['default']; /**
                                      *
                                      * The MIT License (MIT)
                                      *
                                      * Copyright (c) 2015 Invertase
                                      *
                                      * Permission is hereby granted, free of charge, to any person obtaining a copy
                                      * of this software and associated documentation files (the "Software"), to deal
                                      * in the Software without restriction, including without limitation the rights
                                      * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
                                      * copies of the Software, and to permit persons to whom the Software is
                                      * furnished to do so, subject to the following conditions:
                                      *
                                      * The above copyright notice and this permission notice shall be included in all
                                      * copies or substantial portions of the Software.
                                      *
                                      * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
                                      * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
                                      * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
                                      * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
                                      * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
                                      * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
                                      * SOFTWARE.
                                      *
                                      */

// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/defineProperties